use auth::{AuthWindow, WebviewAuthenticator};
use log::info;
use tauri::RunEvent;
use tempfile::NamedTempFile;

use crate::cli::{print_auth_result, Cli};

pub fn authenticate(
  cli: &Cli,
  auth_window: AuthWindow<'static>,
  mut openssl_conf: Option<NamedTempFile>,
) -> anyhow::Result<()> {
  let auth_window = auth_window.with_clean(cli.clean);

  tauri::Builder::default()
    .setup(move |app| {
      let app_handle = app.handle().clone();

      tauri::async_runtime::spawn(async move {
        let auth_result = auth_window.webview_authenticate(&app_handle).await;
        print_auth_result(auth_result);

        // Ensure the app exits after the authentication process
        app_handle.exit(0);
      });

      Ok(())
    })
    .build(tauri::generate_context!())?
    .run(move |_app_handle, event| {
      if let RunEvent::Exit = event {
        if let Some(file) = openssl_conf.take() {
          if let Err(err) = file.close() {
            info!("Error closing OpenSSL config file: {}", err);
          }
        }
      }
    });

  Ok(())
}
